<?php

/**
 * @file
 * Contains module config logic.
 */

/**
 * Global settings form.
 */
function accounting_settings_form($form, &$form_state) {
  $settings = variable_get('accounting_parameters', array());

  $form['tax'] = array(
    '#type' => 'fieldset',
    '#title' => t('Tax data'),
  );
  $calculation_method = isset($settings['calculation_method']) ? $settings['calculation_method'] : 'general';
  $form['tax']['calculation_method'] = array(
    '#type' => 'radios',
    '#title' => t('Tax calculation method'),
    '#options' => array(
      'general' => t('general'),
      'linear' => t('linear'),
    ),
    '#default_value' => $calculation_method,
  );
  $form['tax']['general'] = array(
    '#type' => 'fieldset',
    '#title' => t('General rules tax settings'),
    '#collapsible' => TRUE,
    '#collapsed' => !($calculation_method === 'general'),
  );
  $form['tax']['general']['tax_free_amount'] = array(
    '#type' => 'textfield',
    '#title' => t('Tax reduction amount'),
    '#field_suffix' => 'PLN',
    '#size' => 15,
    '#default_value' => isset($settings['tax_free_amount']) ? $settings['tax_free_amount'] : '',
  );
  $form['tax']['general']['tax_rate_1'] = array(
    '#type' => 'textfield',
    '#title' => t('Tax rate 1'),
    '#field_suffix' => '%',
    '#size' => 15,
    '#default_value' => isset($settings['tax_rate_1']) ? $settings['tax_rate_1'] : '',
  );
  $form['tax']['general']['threshold_1'] = array(
    '#type' => 'textfield',
    '#title' => t('Threshold 1'),
    '#description' => t('Threshold value before tax rate 2'),
    '#field_suffix' => 'PLN',
    '#default_value' => isset($settings['threshold_1']) ? $settings['threshold_1'] : '',
  );
  $form['tax']['general']['tax_rate_2'] = array(
    '#type' => 'textfield',
    '#title' => t('Tax rate 2'),
    '#field_suffix' => '%',
    '#default_value' => isset($settings['tax_rate_2']) ? $settings['tax_rate_2'] : '',
  );
  $form['tax']['linear'] = array(
    '#type' => 'fieldset',
    '#title' => t('Linear rules tax settings'),
    '#collapsible' => TRUE,
    '#collapsed' => !($calculation_method === 'linear'),
  );
  $form['tax']['linear']['tax_rare_linear'] = array(
    '#type' => 'textfield',
    '#title' => t('Tax rate'),
    '#field_suffix' => '%',
    '#size' => 15,
    '#default_value' => isset($settings['tax_rare_linear']) ? $settings['tax_rare_linear'] : '',
  );

  $form['tax']['vat_payer'] = array(
    '#type' => 'checkbox',
    '#title' => t('VAT payer'),
    '#description' => t('Check this checkbox if your company is a VAT payer.'),
    '#default_value' => isset($settings['vat_payer']) ? $settings['vat_payer'] : 0,
  );

  $form['social_insurance'] = array(
    '#type' => 'fieldset',
    '#title' => t('Social insurance data'),
  );
  $form['social_insurance']['health_base'] = array(
    '#type' => 'textfield',
    '#title' => t('Health premium base'),
    '#field_suffix' => 'PLN',
    '#default_value' => isset($settings['health_base']) ? $settings['health_base'] : '',
  );
  $form['social_insurance']['health_deduct'] = array(
    '#type' => 'textfield',
    '#title' => t('Health premium deduct share'),
    '#field_suffix' => '%',
    '#default_value' => isset($settings['health_deduct']) ? $settings['health_deduct'] : '',
  );

  $form['social_insurance']['social_base'] = array(
    '#type' => 'textfield',
    '#title' => t('Social premium base'),
    '#field_suffix' => 'PLN',
    '#default_value' => isset($settings['social_base']) ? $settings['social_base'] : '',
  );

  $form['social_insurance']['social_shares'] = array(
    '#type' => 'fieldset',
    '#title' => t('Social shares for different means'),
  );
  $form['social_insurance']['social_shares']['retirement_share'] = array(
    '#type' => 'textfield',
    '#title' => t('Retirement share'),
    '#field_suffix' => '%',
    '#default_value' => isset($settings['retirement_share']) ? $settings['retirement_share'] : '',
  );
  $form['social_insurance']['social_shares']['pension_share'] = array(
    '#type' => 'textfield',
    '#title' => t('Pension share'),
    '#field_suffix' => '%',
    '#default_value' => isset($settings['pension_share']) ? $settings['pension_share'] : '',
  );
  $form['social_insurance']['social_shares']['illness_share'] = array(
    '#type' => 'textfield',
    '#title' => t('Illness share'),
    '#field_suffix' => '%',
    '#default_value' => isset($settings['illness_share']) ? $settings['illness_share'] : '',
  );
  $form['social_insurance']['social_shares']['accident_share'] = array(
    '#type' => 'textfield',
    '#title' => t('Accident share'),
    '#field_suffix' => '%',
    '#default_value' => isset($settings['accident_share']) ? $settings['accident_share'] : '',
  );
  $form['social_insurance']['social_shares']['labour_share'] = array(
    '#type' => 'textfield',
    '#title' => t('Labour fund'),
    '#field_suffix' => '%',
    '#default_value' => isset($settings['labour_share']) ? $settings['labour_share'] : '',
  );
  $form['social_insurance']['social_shares']['health_share'] = array(
    '#type' => 'textfield',
    '#title' => t('Health share'),
    '#field_suffix' => '%',
    '#default_value' => isset($settings['health_share']) ? $settings['health_share'] : '',
  );

  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save data'),
  );

  return $form;
}

/**
 * Form validate handler.
 */
function accounting_settings_form_validate($form, &$form_state) {
  $skip = array('save', 'form_build_id', 'form_token', 'form_id', 'op');
  $non_numeric = array('calculation_method');
  foreach ($form_state['values'] as $key => $value) {
    if (in_array($key, $skip)) {
      continue;
    }
    $form_state['values'][$key] = strtr($form_state['values'][$key], array(',' => '.'));
    if (!in_array($key, $non_numeric) && !is_numeric($form_state['values'][$key])) {
      form_set_error($key, t('Please provide a numeric value.'));
    }
  }
}

/**
 * Form submit handler.
 */
function accounting_settings_form_submit($form, &$form_state) {
  $skip = array('save', 'form_build_id', 'form_token', 'form_id', 'op');
  $settings = array();
  foreach ($form_state['values'] as $key => $value) {
    if (in_array($key, $skip)) {
      continue;
    }
    $settings[$key] = $value;
  }
  variable_set('accounting_parameters', $settings);
}
