<?php

/**
 * @file
 * Containd module page callbacks and helper functions.
 */

/**
 * Contractor form.
 */
function accounting_contractor_form($form, &$form_state, $cid = 0) {
  if ($cid) {
    $c_data = db_query("SELECT * FROM {accounting_contractors} WHERE cid = :cid", array(':cid' => $cid))->fetchObject();
    if (!$c_data) {
      drupal_set_message(t('error'), 'error');
    }
    $form['#cid'] = $cid;
  }

  $form['contractor'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#default_value' => isset($c_data->contractor) ? $c_data->contractor : '',
    '#required' => TRUE,
  );
  $form['contractor_address'] = array(
    '#type' => 'textarea',
    '#title' => t('Address'),
    '#default_value' => isset($c_data->contractor_address) ? $c_data->contractor_address : '',
    '#required' => TRUE,
  );
  $form['contractor_tid'] = array(
    '#type' => 'textfield',
    '#title' => t('TID'),
    '#default_value' => isset($c_data->contractor_tid) ? $c_data->contractor_tid : '',
  );
  $form['default_descriptions'] = array(
    '#type' => 'textarea',
    '#title' => t('Default description'),
    '#default_value' => isset($c_data->default_descriptions) ? $c_data->default_descriptions : '',
  );

  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save data'),
  );

  $form_state['redirect'] = 'accounting/contractor/list';

  return $form;
}

/**
 * Contractor form validator.
 */
function accounting_contractor_form_validate($form, &$form_state) {
  if (isset($form['contractor'])) {
    my_api_validate_short_text($form['contractor'], $form_state, 128);
  }
  if (isset($form['contractor_fieldset']['contractor'])) {
    my_api_validate_short_text($form['contractor_fieldset']['contractor'], $form_state, 128);
  }
  if (!empty($form_state['values']['contractor_tid'])) {
    $form_state['values']['contractor_tid'] = trim($form_state['values']['contractor_tid']);
    $form_state['values']['contractor_tid'] = strtr($form_state['values']['contractor_tid'], array('-' => ''));
  }
}

/**
 * Contractor form submit handler.
 */
function accounting_contractor_form_submit($form, &$form_state) {
  $ruid = _accounting_get_represented_uid();
  $fields = array(
    'uid' => $ruid,
    'contractor' => $form_state['values']['contractor'],
    'contractor_address' => $form_state['values']['contractor_address'],
    'contractor_tid' => $form_state['values']['contractor_tid'],
    'default_descriptions' => $form_state['values']['default_descriptions'],
  );
  if (isset($form['#cid'])) {
    db_update('accounting_contractors')->fields($fields)->condition('cid', $form['#cid'])->execute();
    drupal_set_message(t('Contractor data saved.'));
  }
  else {
    db_insert('accounting_contractors')->fields($fields)->execute();
    drupal_set_message(t('New contractor has been created.'));
  }
}

/**
 * Contractors list page.
 */
function accounting_contractor_list() {
  $ruid = _accounting_get_represented_uid();
  $rows = db_select('accounting_contractors', 'c')
    ->fields('c', array(
      'cid',
      'contractor',
      'contractor_address',
      'contractor_tid',
    ))
    ->condition('uid', $ruid)
    ->execute()
    ->fetchAllAssoc('cid', PDO::FETCH_ASSOC);
  if (is_array($rows) && !empty($rows)) {
    $header = array(t('Contractor'), t('Address'), t('TID'), t('Actions'));
    foreach ($rows as $cid => $data) {
      unset($rows[$cid]['cid']);
      $actions = array(
        l(t('Edit'), 'accounting/contractor/' . $cid . '/edit'),
        l(t('Delete'), 'accounting/contractor/' . $cid . '/delete'),
      );
      $rows[$cid]['actions'] = theme('item_list', array('items' => $actions));
    }
    return theme('table', array('header' => $header, 'rows' => $rows));
  }
  else {
    return t('You have no contractors at the moment.');
  }
}

/**
 * Contractor delete form.
 */
function accounting_contractor_delete_form($form, &$form_state, $cid = 0) {
  if ($cid) {
    $contractor = db_query("SELECT contractor FROM {accounting_contractors} WHERE cid = :cid", array(':cid' => $cid))->fetchColumn();
    if ($contractor) {
      $form['delete'] = array(
        '#type' => 'submit',
        '#value' => t('Delete contractor'),
        '#prefix' => '<div>' . t('Are you sure you want to delete "!name"?', array('!name' => $contractor)) . '</div>',
      );
      $form['#cid'] = $cid;
    }
  }

  return $form;
}

/**
 * Contractor delete form submit handler.
 */
function accounting_contractor_delete_form_submit($form, &$form_state) {
  $form_state['redirect'] = 'accounting/contractor/list';
  db_delete('accounting_contractors')->condition('cid', $form['#cid'])->execute();
  drupal_set_message(t('Contractor deleted.'));
}

/**
 * Occurrence form.
 */
function accounting_occurrence_form($form, &$form_state, $iid = 0) {
  if ($iid) {
    $i_data = db_query("SELECT * FROM {accounting_dataset} WHERE iid = :iid", array(':iid' => $iid))->fetchObject();
    if (!$i_data) {
      drupal_set_message(t('error'), 'error');
    }
    $form['#iid'] = $iid;
  }
  $form['time'] = array(
    '#title' => t('Occurence date'),
    '#type' => 'textfield',
    '#required' => TRUE,
    '#default_value' => isset($i_data->time) ? $i_data->time : '',
  );
  gp_element_add_date_popup($form['time']);
  $form['code'] = array(
    '#type' => 'textfield',
    '#title' => t('Invoice code'),
    '#default_value' => isset($i_data->code) ? $i_data->code : '',
  );

  // Contractor.
  // AJAX values.
  $ruid = _accounting_get_represented_uid();
  $form['#ajax_values'] = array();
  if (!empty($form_state['values']['contractor'])) {
    $c_data = db_query("SELECT * FROM {accounting_contractors} WHERE contractor = :contractor AND uid = :uid", array(':contractor' => $form_state['values']['contractor'], ':uid' => $ruid))->fetchObject();
    if ($c_data) {
      foreach ($c_data as $key => $value) {
        if ($key == 'default_descriptions') {
          $key = 'description';
        }
        $form['#ajax_values'][$key] = $value;
      }
    }
  }

  $form['contractor_fieldset'] = array(
    '#type' => 'fieldset',
    '#title' => t('Contractor data'),
    '#prefix' => '<div id="contractor-data-wrapper">',
    '#suffix' => '</div>',
  );
  $form['contractor_fieldset']['contractor'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#ajax' => array(
      'wrapper' => 'contractor-data-wrapper',
      'callback' => 'accounting_occurrence_form_ajax',
      'event' => 'blur',
    ),
    '#autocomplete_path' => 'accounting/contractor/fetch/' . $ruid,
    '#default_value' => isset($i_data->contractor) ? $i_data->contractor : '',
  );
  $form['contractor_fieldset']['contractor_address'] = array(
    '#type' => 'textarea',
    '#title' => t('Address'),
    '#default_value' => isset($i_data->contractor_address) ? $i_data->contractor_address : '',
  );
  $form['contractor_fieldset']['contractor_tid'] = array(
    '#type' => 'textfield',
    '#title' => t('TID'),
    '#default_value' => isset($i_data->contractor_tid) ? $i_data->contractor_tid : '',
  );
  $form['contractor_fieldset']['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#default_value' => isset($i_data->description) ? $i_data->description : '',
  );

  if (empty($c_data)) {
    $form['contractor_fieldset']['add_contractor'] = array(
      '#type' => 'checkbox',
      '#title' => t('Add to my contractors'),
    );
  }

  // Income.
  $form['income'] = array(
    '#type' => 'fieldset',
    '#title' => t('Income'),
  );
  $form['income']['income_gs'] = array(
    '#type' => 'textfield',
    '#title' => t('Goods and services income'),
    '#default_value' => isset($i_data->income_gs) ? $i_data->income_gs : '',
  );
  $form['income']['income_other'] = array(
    '#type' => 'textfield',
    '#title' => t('Other income'),
    '#default_value' => isset($i_data->income_other) ? $i_data->income_other : '',
  );

  // Costs.
  $form['costs'] = array(
    '#type' => 'fieldset',
    '#title' => t('Costs'),
  );
  $form['costs']['goods_purchase'] = array(
    '#type' => 'textfield',
    '#title' => t('Goods value'),
    '#default_value' => isset($i_data->goods_purchase) ? $i_data->goods_purchase : '',
    '#description' => t('Trade goods purchase value'),
  );
  $form['costs']['purchase_costs'] = array(
    '#type' => 'textfield',
    '#title' => t('Purchase costs'),
    '#default_value' => isset($i_data->purchase_costs) ? $i_data->purchase_costs : '',
    '#description' => t('Trade goods purchase additional costs'),
  );
  $form['costs']['cost_salaries'] = array(
    '#type' => 'textfield',
    '#title' => t('Salaries'),
    '#default_value' => isset($i_data->cost_salaries) ? $i_data->cost_salaries : '',
  );
  $form['costs']['cost_other'] = array(
    '#type' => 'textfield',
    '#title' => t('Other costs'),
    '#default_value' => isset($i_data->cost_other) ? $i_data->cost_other : '',
  );

  // VAT.
  $settings = variable_get('accounting_parameters');
  if (!empty($settings['vat_payer'])) {
    $form['income']['vat_add'] = array(
      '#type' => 'textfield',
      '#title' => t('VAT'),
      '#default_value' => (isset($i_data->vat) && $i_data->vat > 0) ? $i_data->vat : '',
    );
    $form['costs']['vat_subtract'] = array(
      '#type' => 'textfield',
      '#title' => t('VAT'),
      '#default_value' => (isset($i_data->vat) && $i_data->vat < 0) ? -$i_data->vat : '',
    );
  }

  // Remarks.
  $form['remarks'] = array(
    '#type' => 'textarea',
    '#title' => t('Remarks'),
    '#default_value' => isset($i_data->remarks) ? $i_data->remarks : '',
  );

  // Attached document.
  $form['fid'] = array(
    '#type' => 'managed_file',
    '#title' => t('Document file'),
    '#default_value' => isset($i_data->fid) ? $i_data->fid : '',
  );

  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save data'),
  );

  if ($iid) {
    $form['clone'] = array(
      '#type' => 'submit',
      '#value' => t('Clone invoice'),
    );
  }

  return $form;
}

/**
 * AJAX callback.
 */
function accounting_occurrence_form_ajax($form, &$form_state) {
  $ruid = _accounting_get_represented_uid();
  if (accounting_fetch_user_contractors($ruid, $form_state['values']['contractor'], FALSE)) {
    foreach ($form['#ajax_values'] as $key => $value) {
      $form['contractor_fieldset'][$key]['#value'] = $value;
    }

    return $form['contractor_fieldset'];
  }
}

/**
 * Validate occurrence form.
 */
function accounting_occurrence_form_validate($form, &$form_state) {

  my_api_validate_short_text($form['code'], $form_state, 128);

  accounting_contractor_form_validate($form, $form_state);

  my_api_validate_number($form['income']['income_gs'], $form_state);
  my_api_validate_number($form['income']['income_other'], $form_state);
  my_api_validate_number($form['costs']['goods_purchase'], $form_state);
  my_api_validate_number($form['costs']['purchase_costs'], $form_state);
  my_api_validate_number($form['costs']['cost_salaries'], $form_state);
  my_api_validate_number($form['costs']['cost_other'], $form_state);
  if (isset($form['income']['vat_add'])) {
    my_api_validate_number($form['income']['vat_add'], $form_state);
  }
  if (isset($form['costs']['vat_subtract'])) {
    my_api_validate_number($form['costs']['vat_subtract'], $form_state);
  }

  foreach (array(
    'income_gs',
    'income_other',
    'goods_purchase',
    'purchase_costs',
    'cost_salaries',
    'cost_other',
  ) as $field) {
    if (isset($form_state['values'][$field]) && $form_state['values'][$field] == '') {
      $form_state['values'][$field] = 0;
    }
  }

  // Preprocess VAT.
  foreach (array('vat_add', 'vat_subtract') as $vat_type) {
    if (empty($form_state['values'][$vat_type])) {
      $form_state['values'][$vat_type] = 0;
    }
  }
  $form_state['values']['vat'] = $form_state['values']['vat_add'] - $form_state['values']['vat_subtract'];
  unset($form_state['values']['vat_add']);
  unset($form_state['values']['vat_subtract']);
}

/**
 * Helper function to save attachment file.
 */
function _accounting_process_attachment($fid, $iid, $op, $time = REQUEST_TIME) {
  $file = file_load($fid);
  if ($file) {
    if ($op == 'create') {
      $directory = 'private://accounting/' . date('Y_m', $time);
      if (file_prepare_directory($directory, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS)) {
        $filepath = $directory . '/' . $file->filename;
        $file = file_move($file, $filepath);
        // Change status to permanent.
        $file->status = FILE_STATUS_PERMANENT;
        // Save.
        file_save($file);
        // Record that the module (in this example, user module)
        // is using the file.
        file_usage_add($file, 'accounting', 'occurrence', $iid);
        return TRUE;
      }
    }
    else {
      file_usage_delete($file, 'accounting', 'occurrence', $iid);
      if (empty(file_usage_list($file))) {
        file_delete($file);
      }
      return TRUE;
    }
  }
  return FALSE;
}

/**
 * Occurrence form submit handler.
 */
function accounting_occurrence_form_submit($form, &$form_state) {
  $ruid = _accounting_get_represented_uid();
  $form_state['redirect'] = 'accounting/occurrence/list';
  $op = end($form_state['triggering_element']['#parents']);

  $fields = array(
    'uid' => $ruid,
    'time' => $form_state['values']['time'],
    'code' => $form_state['values']['code'],
    'contractor' => $form_state['values']['contractor'],
    'contractor_address' => $form_state['values']['contractor_address'],
    'contractor_tid' => $form_state['values']['contractor_tid'],
    'description' => $form_state['values']['description'],
    'income_gs' => $form_state['values']['income_gs'],
    'income_other' => $form_state['values']['income_other'],
    'income_total' => $form_state['values']['income_gs'] + $form_state['values']['income_other'],
    'goods_purchase' => $form_state['values']['goods_purchase'],
    'purchase_costs' => $form_state['values']['purchase_costs'],
    'cost_salaries' => $form_state['values']['cost_salaries'],
    'cost_other' => $form_state['values']['cost_other'],
    'cost_total' => $form_state['values']['cost_salaries'] + $form_state['values']['cost_other'],
    'vat' => $form_state['values']['vat'],
    'remarks' => $form_state['values']['remarks'],
    'fid' => $form_state['values']['fid'],
  );

  // Attachment logic.
  if (isset($form['#iid']) && $op === 'save') {
    $old_fid = db_query("SELECT fid FROM {accounting_dataset} WHERE iid = :iid", array(':iid' => $form['#iid']))->fetchField();
  }
  else {
    $old_fid = 0;
  }
  $new_fid = $form_state['values']['fid'];
  $file_operations = array();
  if (empty($new_fid) && !empty($old_fid)) {
    $file_operations['delete'] = $old_fid;
  }
  elseif (!empty($new_fid) && $new_fid != $old_fid) {
    if (!empty($old_fid)) {
      $file_operations['delete'] = $old_fid;
    }
    $file_operations['create'] = $new_fid;
  }

  if (isset($form['#iid']) && $op === 'save') {
    $iid = $form['#iid'];
    db_update('accounting_dataset')->fields($fields)->condition('iid', $form['#iid'])->execute();
    drupal_set_message(t('Occurrence data saved.'));
  }
  else {
    $uiid = db_select('accounting_dataset', 'ad')
      ->fields('ad', array('uiid'))
      ->condition('uid', $ruid)
      ->orderBy('uiid', 'DESC')
      ->range(0, 1)
      ->execute()
      ->fetchField();
    if (!$uiid) {
      $uiid = 1;
    }
    else {
      $uiid++;
    }
    $fields['uiid'] = $uiid;
    $iid = db_insert('accounting_dataset')->fields($fields)->execute();
    drupal_set_message(t('New occurrence has been created. Serial number: !number.', array('!number' => $uiid)));
  }

  // Process files.
  foreach ($file_operations as $op => $fid) {
    _accounting_process_attachment($fid, $iid, $op, $form_state['values']['time']);
  }

  // Save contractor data.
  if (isset($form_state['values']['add_contractor']) && $form_state['values']['add_contractor']) {
    $form_state['values']['default_descriptions'] = $form_state['values']['description'];
    accounting_contractor_form_submit($form, $form_state);
  }

}

/**
 * Delete occurrence form.
 */
function accounting_occurrence_delete_form($form, &$form_state, $iid = 0) {
  if ($iid) {
    $occurrence = db_query("SELECT description FROM {accounting_dataset} WHERE iid = :iid", array(':iid' => $iid))->fetchColumn();
    if ($occurrence) {
      $form['delete'] = array(
        '#type' => 'submit',
        '#value' => t('Delete occurrence'),
        '#prefix' => '<div>' . t('Are you sure you want to delete "!name"?', array('!name' => $occurrence)) . '</div>',
      );
      $form['#iid'] = $iid;
    }
  }

  return $form;
}

/**
 * Delete occurrence form submit handler.
 */
function accounting_occurrence_delete_form_submit($form, &$form_state) {
  $form_state['redirect'] = 'accounting/occurrence/list';
  db_delete('accounting_dataset')->condition('iid', $form['#iid'])->execute();
  drupal_set_message(t('Occurrence deleted.'));
}

/**
 * Prepare nav links.
 */
function _accounting_prepare_month_nav_links($month, $year) {
  $nav = array(
    'prev' => array(
      'm' => $month - 1,
      'y' => $year,
    ),
    'next' => array(
      'm' => $month + 1,
      'y' => $year,
    ),
  );
  if ($month == 1) {
    $nav['prev']['m'] = 12;
    $nav['prev']['y']--;
  }
  elseif ($month == 12) {
    $nav['next']['m'] = 1;
    $nav['next']['y']++;
  }

  $renderable = array(
    '#theme' => 'item_list',
    '#items' => array(),
    '#attributes' => array(
      'class' => array('accounting-nav'),
    ),
  );

  $destination = drupal_get_destination();

  // Account for the global /accounting path.
  if ($destination['destination'] == 'accounting' || $destination['destination'] == '/accounting') {
    $path = 'accounting/occurrence/list';
  }
  else {
    $path = substr($destination['destination'], 0, strpos($destination['destination'], '/list') + 5);
  }

  foreach ($nav as $type => $data) {
    $item = array(
      '#theme' => 'link',
      '#path' => $path . '/' . $data['y'] . '/' . $data['m'],
      '#options' => array(
        'attributes' => array(),
        'html' => FALSE,
      ),
    );
    if ($type == 'prev') {
      $item['#text'] = '<< ' . $data['m'] . '.' . $data['y'];
    }
    else {
      $item['#text'] = $data['m'] . '.' . $data['y'] . ' >>';
    }
    $renderable['#items'][] = render($item);
  }

  return $renderable;
}

/**
 * Prepare action links.
 */
function _accounting_prepare_action_links($item) {
  $renderable = array(
    '#theme' => 'item_list',
    '#items' => array(),
  );

  $destination = drupal_get_destination();
  $links = array(
    'accounting/occurrence/' . $item->iid . '/edit' => t('Edit'),
    'accounting/occurrence/' . $item->iid . '/delete' => t('Delete'),
  );
  foreach ($links as $uri => $text) {
    $link = array(
      '#theme' => 'link',
      '#path' => $uri,
      '#text' => $text,
      '#options' => array(
        'attributes' => array(),
        'html' => FALSE,
        'query' => $destination,
      ),
    );

    $renderable['#items'][] = render($link);
  }

  if (!empty($item->uri)) {
    $link = array(
      '#theme' => 'link',
      '#path' => file_create_url($item->uri),
      '#text' => t('Download'),
      '#options' => array(
        'attributes' => array(),
        'html' => FALSE,
        'query' => $destination,
      ),
    );

    $renderable['#items'][] = render($link);
  }

  return render($renderable);
}

/**
 * Monthly view of data sheet - the actual thing.
 */
function accounting_occurrence_list($year = FALSE, $month = FALSE, $print = FALSE) {
  if (!$year) {
    $year = date('Y');
  }
  if (!$month) {
    $month = date('m');
  }
  $month = (int) $month;

  $ruid = _accounting_get_represented_uid();
  $occurrences = accounting_get_occurrences($year, $month, $ruid);

  $fields = array(
    'uiid',
    'time',
    'code',
    'contractor',
    'contractor_address',
    'description',
    'income_gs',
    'income_other',
    'income_total',
    'goods_purchase',
    'purchase_costs',
    'cost_salaries',
    'cost_other',
    'cost_total',
    'remarks',
  );

  // Header rows.
  $rows = array(
    array(
      array(
        'header' => TRUE,
        'data' => t('No', array(), array('context' => 'number')),
        'rowspan' => 2,
      ),
      array(
        'header' => TRUE,
        'data' => t('Occurrence date'),
        'rowspan' => 2,
      ),
      array(
        'header' => TRUE,
        'data' => t('Accounting document no'),
        'rowspan' => 2,
      ),
      array(
        'header' => TRUE,
        'data' => t('Contractor'),
        'colspan' => 2,
      ),
      array(
        'header' => TRUE,
        'data' => t('Occurrence description'),
        'rowspan' => 2,
      ),
      array(
        'header' => TRUE,
        'data' => t('Income'),
        'colspan' => 3,
      ),
      array(
        'header' => TRUE,
        'data' => t('Trade goods purchase according to purchase prices'),
        'rowspan' => 2,
      ),
      array(
        'header' => TRUE,
        'data' => t('Additional purchase costs'),
        'rowspan' => 2,
      ),
      array(
        'header' => TRUE,
        'data' => t('Costs'),
        'colspan' => 3,
      ),
      array(
        'header' => TRUE,
        'data' => t('Remarks'),
        'rowspan' => 2,
      ),
    ),
    array(
      array(
        'header' => TRUE,
        'data' => t('Name and family (company)'),
      ),
      array(
        'header' => TRUE,
        'data' => t('address'),
      ),
      array(
        'header' => TRUE,
        'data' => t("Sold goods and services' value"),
      ),
      array(
        'header' => TRUE,
        'data' => t('Other income'),
      ),
      array(
        'header' => TRUE,
        'data' => t('Total income'),
      ),
      array(
        'header' => TRUE,
        'data' => t('Salary in cash and in kind'),
      ),
      array(
        'header' => TRUE,
        'data' => t('Other expenses'),
      ),
      array(
        'header' => TRUE,
        'data' => t('Total costs'),
      ),
    ),
    array(
      'class' => array('numbers'),
      'data' => array(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15),
    ),
  );
  if (!$print) {
    $rows[0][] = array('header' => TRUE, 'data' => t('Actions'), 'rowspan' => 2);
    $rows[2]['data'][] = 16;
  }

  $sums = array(
    'income_gs' => 0,
    'income_other' => 0,
    'income_total' => 0,
    'goods_purchase' => 0,
    'purchase_costs' => 0,
    'cost_salaries' => 0,
    'cost_other' => 0,
    'cost_total' => 0,
  );
  $index = count($rows);
  foreach ($occurrences as $delta => $item) {
    unset($item->uid);
    unset($item->contractor_tid);
    $item->time = date('d.m.Y', $item->time);
    foreach ($fields as $field) {
      $rows[$index][] = $item->{$field};
    }
    if (!$print) {
      $rows[$index][] = _accounting_prepare_action_links($item);
    }
    foreach ($sums as $key => $value) {
      $sums[$key] += $item->{$key};
    }
    $index++;
  }

  // Sum row.
  $rows[$index][] = array(
    'data' => '',
    'colspan' => 5,
    'class' => array('empty'),
  );
  $rows[$index][] = array(
    'data' => t('Page sum'),
    'header' => TRUE,
  );
  foreach ($sums as $key => $value) {
    $rows[$index][] = array(
      'data' => $value,
      'class' => array('sum'),
    );
  }
  if ($print) {
    $rows[$index][] = array(
      'data' => '',
      'class' => array('empty'),
    );
  }
  else {
    $rows[$index][] = array(
      'data' => '',
      'colspan' => 2,
      'class' => array('empty'),
    );
  }

  $table = theme('table', array(
    'header' => array(),
    'rows' => $rows,
    'attributes' => array('class' => array('accounting-table')),
  ));

  drupal_add_css(drupal_get_path('module', 'accounting') . '/accounting.css');
  if ($print) {
    $month_names = _accounting_get_month_names();
    module_invoke('admin_menu', 'suppress');
    $header = '<h1>PKPiR, rok: ' . $year . ', miesiąc: ' . $month_names[$month] . '</h1>';
    return $header . $table;
  }
  else {
    $action_links = array(
      l(t('Add new occurrence'), 'accounting/occurrence/add', array('query' => drupal_get_destination())),
      l(t('Print data table'), 'accounting/occurrence/list/' . $year . '/' . $month . '/print', array(
        'attributes' => array('target' => '_blank'),
      )),
    );

    $reorder_form = drupal_get_form('accounting_reorder_form', 'accounting/occurrence/list/' . $year . '/' . $month);
    $nav_links = _accounting_prepare_month_nav_links($month, $year);
    return
      render($nav_links) .
      theme('item_list', array(
        'items' => $action_links,
        'attributes' => array(
          'class' => array('accounting-links'),
        ),
      )) .
      $table .
      render($reorder_form);
  }
}

/**
 * VAT.
 */
function accounting_vat_page($year = FALSE, $month = FALSE) {
  if (!$year) {
    $year = date('Y');
  }
  if (!$month) {
    $month = date('m');
  }
  $month = (int) $month;

  $ruid = _accounting_get_represented_uid();
  $occurrences = accounting_get_occurrences($year, $month, $ruid, TRUE);

  $renderable = array(
    'links' => _accounting_prepare_month_nav_links($month, $year),
    'add_link' => array(
      '#theme' => 'link',
      '#path' => 'accounting/occurrence/add',
      '#text' => t('Add new occurrence'),
      '#options' => array(
        'attributes' => array(),
        'html' => FALSE,
        'query' => drupal_get_destination(),
      ),
    ),
    'table' => array(
      '#theme' => 'table',
      '#header' => array(
        'no' => t('No', array(), array('context' => 'number')),
        'date' => t('Occurrence date'),
        'doc_no' => t('Accounting document no'),
        'name' => t('Name and family (company)'),
        'address' => t('address'),
        'description' => t('Occurrence description'),
        'vat' => t('VAT'),
        'remarks' => t('Remarks'),
        'actions' => t('Actions'),
      ),
      '#rows' => array(),
      '#empty' => t('There are no VAT items for this month yet.')
    ),
  );

  if (!empty($occurrences)) {
    $total_vat = 0;
    foreach ($occurrences as $item) {
      $renderable['table']['#rows'][] = array(
        'no' => $item->uiid,
        'date' => date('d.m.Y', $item->time),
        'doc_no' => $item->code,
        'name' => $item->contractor,
        'address' => $item->contractor_address,
        'description' => $item->description,
        'vat' => $item->vat,
        'remarks' => $item->remarks,
        'actions' => _accounting_prepare_action_links($item),
      );
      $total_vat += $item->vat;
    }

    // Sum row.
    $renderable['table']['#rows'][] = array(
      array(
        'data' => t('Sum'),
        'header' => TRUE,
        'colspan' => 6,
      ),
      array(
        'data' => $total_vat,
        'class' => array('sum'),
      ),
      array(
        'data' => '',
        'class' => array('sum'),
        'colspan' => 2,
      ),
    );
  }

  return $renderable;
}

/**
 * Annual balance.
 */
function accounting_annual_balance($year = FALSE) {
  if (!$year) {
    $year = date('Y', REQUEST_TIME);
  }
  drupal_set_title(t('Accounting balance for !year', array('!year' => $year)));
  drupal_add_css(drupal_get_path('module', 'accounting') . '/accounting.css');
  $nav_links = array(
    l('<< ' . ($year - 1), 'accounting/annual-balance/' . ($year - 1)),
    l(($year + 1) . ' >>', 'accounting/annual-balance/' . ($year + 1)),
  );
  $premium_info = accounting_premium_info();
  $annual_form = drupal_get_form('accounting_annual_form', $year);
  return '<div id="accounting">' . theme('item_list', array('items' => $nav_links, 'attributes' => array('class' => array('accounting-nav')))) . $premium_info . render($annual_form) . '</div>';
}

/**
 * Generate premium info.
 */
function accounting_premium_info() {
  $settings = variable_get('accounting_parameters');
  $param_rows = array(
    array(
      array(
        'data' => t('Social premium base'),
        'header' => TRUE,
      ),
      $settings['social_base'] . ' PLN',
    ),
    array(
      array(
        'data' => t('Health premium base'),
        'header' => TRUE,
      ),
      $settings['health_base'] . ' PLN',
    ),
  );

  $premium = array(
    'Retirement' => round($settings['social_base'] * $settings['retirement_share'] / 100, 2),
    'Pension' => round($settings['social_base'] * $settings['pension_share'] / 100, 2),
    'Illness' => round($settings['social_base'] * $settings['illness_share'] / 100, 2),
    'Accidental' => round($settings['social_base'] * $settings['accident_share'] / 100, 2),
    'Labour' => round($settings['social_base'] * $settings['labour_share'] / 100, 2),
    'Health' => round($settings['health_base'] * $settings['health_share'] / 100, 2),
    'Health deduct' => round($settings['health_base'] * $settings['health_deduct'] / 100, 2),
  );
  $premium['Sum1'] = $premium['Retirement'] + $premium['Pension'] + $premium['Accidental'];
  $premium['Sum2'] = $premium['Retirement'] + $premium['Pension'] + $premium['Illness'] + $premium['Accidental'];
  $premium['Sum3'] = $premium['Sum1'] + $premium['Health'] + $premium['Labour'];
  $premium['Sum4'] = $premium['Sum2'] + $premium['Health'] + $premium['Labour'];

  foreach ($premium as $key => $value) {
    $premium[$key] = number_format($value, 2, ',', ' ');
  }

  $to_pay_total = t('@no-illness-total (@illness-total with illness share)', array(
    '@no-illness-total' => $premium['Sum3'],
    '@illness-total' => $premium['Sum4'],
  ));

  $premium_rows = array(
    array(
      array('data' => t('Insurance'), 'header' => TRUE),
      t('Retirement'),
      t('Pension'),
      t('Illness'),
      t('Accidental'),
      t('Social sum without Illness share'),
      t('Social sum with Illness share'),
      t('Labour fund'),
      t('Health'),
      t('Health deduct'),
    ),
    array(
      array('data' => t('Base share'), 'header' => TRUE),
      $settings['retirement_share'],
      $settings['pension_share'],
      $settings['illness_share'],
      $settings['accident_share'],
      $settings['retirement_share'] + $settings['pension_share'] + $settings['accident_share'],
      $settings['retirement_share'] + $settings['pension_share'] + $settings['illness_share'] + $settings['accident_share'],
      $settings['labour_share'],
      $settings['health_share'],
      $settings['health_deduct'],
    ),
    array(
      array('data' => t('Pension amount'), 'header' => TRUE),
      $premium['Retirement'],
      $premium['Pension'],
      $premium['Illness'],
      $premium['Accidental'],
      $premium['Sum1'],
      $premium['Sum2'],
      $premium['Labour'],
      $premium['Health'],
      $premium['Health deduct'],
    ),
    array(
      array('data' => t('Total to pay'), 'header' => TRUE),
      array('data' => $to_pay_total, 'colspan' => 9),
    ),
  );

  return theme('table', array('rows' => $param_rows)) . theme('table', array('rows' => $premium_rows));
}

/**
 * Helper function to get month names.
 */
function _accounting_get_month_names() {
  return array(
    1 => t('January'),
    2 => t('February'),
    3 => t('March'),
    4 => t('April'),
    5 => t('May'),
    6 => t('June'),
    7 => t('July'),
    8 => t('August'),
    9 => t('September'),
    10 => t('October'),
    11 => t('November'),
    12 => t('December'),
  );
}

/**
 * Annual form builder.
 */
function accounting_annual_form($form, &$form_state, $year) {
  $form['#year'] = $year;
  $form['#months'] = _accounting_get_month_names();

  $ruid = _accounting_get_represented_uid();
  $annual_data = db_select('accounting_annual', 'a')
    ->fields('a')
    ->condition('uid', $ruid)
    ->condition('year', $year)
    ->execute()
    ->fetchAllAssoc('month', PDO::FETCH_ASSOC);
  if (!$annual_data) {
    $annual_data = array();
  }

  $form['#fields'] = array(
    'si_social' => array('title' => t('Social deduce')),
    'si_health' => array('title' => t('Health deduce')),
    'sales' => array('title' => t('Income'), 'disabled' => TRUE),
    'outcome' => array('title' => t('Outcome'), 'disabled' => TRUE),
    'income' => array('title' => t('Profit'), 'disabled' => TRUE),
    'advance' => array('title' => t('Advance'), 'disabled' => TRUE),
    'to_pay' => array('title' => t('To pay'), 'disabled' => TRUE),
    'paid' => array('title' => t('Paid')),
  );

  // Display VAT fields if applicable.
  $settings = variable_get('accounting_parameters');
  if (!empty($settings['vat_payer'])) {
    $form['#fields']['vat_to_pay'] = array('title' => t('VAT to pay'), 'disabled' => TRUE);
    $form['#fields']['vat_paid'] = array('title' => t('VAT paid'));
  }

  $form['#column_sums'] = array();
  foreach ($form['#months'] as $no => $name) {
    $form[$no] = array(
      '#type' => 'fieldset',
      '#title' => $name,
    );
    foreach ($form['#fields'] as $field => $data) {
      $form[$no][$field . '_' . $no] = array(
        '#type' => 'textfield',
        '#title' => $data['title'],
        '#size' => 15,
        '#default_value' => isset($annual_data[$no]) ? $annual_data[$no][$field] : 0,
      );

      if (!isset($form['#column_sums'][$field])) {
        $form['#column_sums'][$field] = 0;
      }
      $form['#column_sums'][$field] += $form[$no][$field . '_' . $no]['#default_value'];

      if (isset($data['disabled']) && $data['disabled']) {
        $form[$no][$field . '_' . $no]['#attributes'] = array('disabled' => TRUE);
      }
    }
  }

  // Allow to save changes only until 30th day of January next year.
  if ($year === date('Y', (REQUEST_TIME - 30 * 86400))) {
    $form['save'] = array(
      '#type' => 'submit',
      '#value' => t('Save / Recalculate'),
    );
  }

  return $form;
}

/**
 * Annual form validator.
 */
function accounting_annual_form_validate($form, &$form_state) {
  foreach ($form['#months'] as $no => $name) {
    foreach ($form['#fields'] as $field => $data) {
      if (empty($form_state['values'][$field . '_' . $no])) {
        $form_state['values'][$field . '_' . $no] = 0;
      }
      else {
        $form_state['values'][$field . '_' . $no] = strtr($form_state['values'][$field . '_' . $no], array(',' => '.'));
        if (!is_numeric($form_state['values'][$field . '_' . $no])) {
          form_set_error($field . '_' . $no, 'All values in the table must be numeric.');
        }
      }
    }
  }
}

/**
 * Annual form submit handler.
 */
function accounting_annual_form_submit($form, &$form_state) {
  $ruid = _accounting_get_represented_uid();
  $params = variable_get('accounting_parameters', array());
  $month_data = array();
  $month_data[0]['increasing'] = array(
    'si_social' => 0,
    'si_health' => 0,
    'income' => 0,
    'tax' => 0,
    'paid' => 0,
    'vat_paid' => 0,
    'vat_to_pay' => 0,
  );
  foreach ($form['#months'] as $no => $name) {
    $occurrences = accounting_get_occurrences($form['#year'], $no, $ruid);
    $month_data[$no]['monthly'] = array(
      'income' => 0,
      'outcome' => 0,
      'sales' => 0,
      'si_social' => $form_state['values']['si_social_' . $no],
      'si_health' => $form_state['values']['si_health_' . $no],
      'paid' => $form_state['values']['paid_' . $no],
      'vat_paid' => isset($form_state['values']['vat_paid_' . $no]) ? $form_state['values']['vat_paid_' . $no] : 0,
      'vat_to_pay' => 0,
    );

    $month_data[$no]['monthly']['vat_to_pay'] = 0;
    foreach ($occurrences as $occurrence) {
      $month_data[$no]['monthly']['sales'] += $occurrence->income_total;
      $month_data[$no]['monthly']['outcome'] += $occurrence->goods_purchase + $occurrence->purchase_costs + $occurrence->cost_total;
      $month_data[$no]['monthly']['vat_to_pay'] += $occurrence->vat;
    }
    $month_data[$no]['monthly']['vat_to_pay'] = round($month_data[$no]['monthly']['vat_to_pay'], 0);

    $month_data[$no]['monthly']['income'] = $month_data[$no]['monthly']['sales'] - $month_data[$no]['monthly']['outcome'];

    // Increasing values.
    $month_data[$no]['increasing'] = array(
      'si_social' => $month_data[$no - 1]['increasing']['si_social'] + $form_state['values']['si_social' . '_' . $no],
      'si_health' => $month_data[$no - 1]['increasing']['si_health'] + $form_state['values']['si_health' . '_' . $no],
      'income' => $month_data[$no - 1]['increasing']['income'] + $month_data[$no]['monthly']['income'],
      'paid' => $month_data[$no - 1]['increasing']['paid'] + $month_data[$no]['monthly']['paid'],
      'vat_paid' => $month_data[$no - 1]['increasing']['vat_paid'] + $month_data[$no]['monthly']['vat_paid'],
      'vat_to_pay' => $month_data[$no - 1]['increasing']['vat_to_pay'] + $month_data[$no]['monthly']['vat_to_pay'],
    );
    $tax_base = round($month_data[$no]['increasing']['income'] - $month_data[$no]['increasing']['si_social'], 0);

    $month_data[$no]['increasing']['tax'] = round(accounting_calculate_tax($tax_base, $params) - $month_data[$no]['increasing']['si_health'], 0);
    if ($month_data[$no]['increasing']['tax'] < 0) {
      $month_data[$no]['increasing']['tax'] = 0;
    }

    $month_data[$no]['monthly']['advance'] = $month_data[$no]['increasing']['tax'] - $month_data[$no - 1]['increasing']['tax'];

    $month_data[$no]['monthly']['to_pay'] = $month_data[$no]['increasing']['tax'] - $month_data[$no]['increasing']['paid'];
    if ($month_data[$no]['monthly']['to_pay'] < 0) {
      $month_data[$no]['monthly']['to_pay'] = 0;
    }

    $month_data[$no]['monthly']['vat_to_pay'] = $month_data[$no]['increasing']['vat_to_pay'] - $month_data[$no]['increasing']['vat_paid'];

    $keys = array('uid' => $ruid, 'year' => $form['#year'], 'month' => $no);
    $fields = $month_data[$no]['monthly'] + $keys;
    db_merge('accounting_annual')->key($keys)->fields($fields)->execute();
  }

  drupal_set_message(t('Annual data has been recalculated and saved.'));
}

/**
 * Occurrence reorder form.
 */
function accounting_reorder_form($form, &$form_state, $destination) {
  $form_state['#destination'] = $destination;
  $form['reorder'] = array(
    '#type' => 'submit',
    '#value' => t('Reorder occurrences'),
  );
  $form['description'] = array(
    '#markup' => t('Occurrences will be ordered by date and assigned new numbers (view last report !link).', array(
      '!link' => l(t('>>'), 'accounting/ordering-report'),
    )),
  );
  return $form;
}

/**
 * Occurrence reorder form submit handler.
 */
function accounting_reorder_form_submit($form, &$form_state) {
  $batch = array(
    'title' => t('Reordering occurrences'),
    'operations' => array(
      array('_accounting_reorder_occurrences', array()),
    ),
    'finished' => '_accounting_reorder_occurrences_finish',
    'init_message' => t('Starting operation..'),
    'progress_message' => '',
    'error_message' => t('An error occurred.'),
  );

  batch_set($batch);
  batch_process($form_state['#destination']);
}

/**
 * Last reorder report page callback.
 */
function accounting_last_report_page() {
  $ruid = _accounting_get_represented_uid();
  $report_data = variable_get('accounting_last_reorder_report_' . $ruid, FALSE);
  if (!$report_data) {
    drupal_set_message(t('No report data available.'));
    return '';
  }
  else {
    $iids = array();
    foreach ($report_data as $item) {
      $iids[] = $item['iid'];
    }
    $items = db_select('accounting_dataset', 'd')->fields('d')->condition('uid', $ruid)->condition('iid', $iids, 'IN')->orderBy('time', 'ASC')->execute()->fetchAllAssoc('iid', PDO::FETCH_ASSOC);
    $table_vars['header'] = array(
      t('Occurence date'),
      t('Accounting document no'),
      t('Name and family (company)'),
      t('Address'),
      t('Occurrence description'),
      t('Old ID'),
      t('New ID'),
    );
    $table_vars['rows'] = array();
    foreach ($report_data as $item) {
      $iid = $item['iid'];
      $table_vars['rows'][] = array(
        date('d-m-Y', $items[$iid]['time']),
        $items[$iid]['code'],
        $items[$iid]['contractor'],
        $items[$iid]['contractor_address'],
        $items[$iid]['description'],
        $item['old_uiid'],
        $items[$iid]['uiid'],
      );
    }
    return theme('table', $table_vars);
  }
}
