<?php

/**
 * @file
 * Provides integration with graber_cart accounting documents.
 */

/**
 * Implements hook_form_FORM_ID_alter().
 */
function accounting_form_gad_document_form_alter(&$form, &$form_state) {
  $ruid = _accounting_get_represented_uid();
  if (isset($form['billing']['company'])) {
    $element = &$form['billing']['company'];
  }
  elseif (isset($form['billing']['name'])) {
    $element = &$form['billing']['name'];
  }

  $element += array(
    '#autocomplete_path' => 'accounting/contractor/fetch/' . $ruid,
    '#ajax' => array(
      'wrapper' => 'contractor-data-wrapper',
      'callback' => 'accounting_integration_insert_form_values',
      'event' => 'blur',
    ),
  );
}

/**
 * AJAX callback to populate form values.
 */
function accounting_integration_insert_form_values($form, $form_state) {
  if (isset($form_state['values']['billing']['company'])) {
    $input = $form_state['values']['billing']['company'];
  }
  elseif (isset($form_state['values']['billing']['name'])) {
    $input = $form_state['values']['billing']['name'];
  }
  if (isset($input)) {
    $ruid = _accounting_get_represented_uid();
    $contractor = db_select('accounting_contractors', 'c')
      ->fields('c')
      ->condition('c.uid', $ruid)
      ->condition('contractor', '%' . db_like($input) . '%', 'LIKE')
      ->execute()
      ->fetchAssoc();

    $values = array();

    if (strlen($contractor['contractor_tid']) == 10) {
      $values['tin'] = substr($contractor['contractor_tid'], 0, 3) . '-' . substr($contractor['contractor_tid'], 3, 3) . '-' . substr($contractor['contractor_tid'], 6, 2) . '-' . substr($contractor['contractor_tid'], 8, 2);
    }
    else {
      $values['tin'] = $contractor['contractor_tid'];
    }

    $address_parts = explode(';', $contractor['contractor_address']);
    if (count($address_parts) == 1) {
      $address_parts = explode(',', $contractor['contractor_address']);
    }
    if (count($address_parts) == 2) {
      $values['address'] = trim($address_parts[0]);
      $post_city = explode(' ', trim($address_parts[1]));
      $values['postal_code'] = array_shift($post_city);
      $values['city'] = implode(' ', $post_city);
    }
    else {
      $values['address'] = $contractor['contractor_address'];
    }

    $commands = array();
    foreach ($values as $field => $value) {
      $commands[] = ajax_command_invoke('#edit-billing-' . (strtr($field, array('_' => '-'))), 'val', array($value));
    }
    return array('#type' => 'ajax', '#commands' => $commands);
  }
}
